/*
 *
 * (C) Copyright IBM Corp. 1998-2011. All Rights Reserved.
 *
 * WARNING: THIS FILE IS MACHINE GENERATED. DO NOT HAND EDIT IT UNLESS
 * YOU REALLY KNOW WHAT YOU'RE DOING.
 *
 * Generated on: 11/01/2011 04:08:11 PM PDT
 */

#ifndef __SCRIPTANDLANGUAGES_H
#define __SCRIPTANDLANGUAGES_H

#include "LETypes.h"

/**
 * \file
 * \internal
 */

U_NAMESPACE_BEGIN

const LETag zyyyScriptTag = 0x7A797979; /* 'zyyy' (COMMON) */
const LETag zinhScriptTag = 0x7A696E68; /* 'zinh' (INHERITED) */
const LETag arabScriptTag = 0x61726162; /* 'arab' (ARABIC) */
const LETag armnScriptTag = 0x61726D6E; /* 'armn' (ARMENIAN) */
const LETag bengScriptTag = 0x62656E67; /* 'beng' (BENGALI) */
const LETag bng2ScriptTag = 0x626E6732; /* 'bng2' (BENGALI v.2) (manually added) */
const LETag bopoScriptTag = 0x626F706F; /* 'bopo' (BOPOMOFO) */
const LETag cherScriptTag = 0x63686572; /* 'cher' (CHEROKEE) */
const LETag coptScriptTag = 0x636F7074; /* 'copt' (COPTIC) */
const LETag cyrlScriptTag = 0x6379726C; /* 'cyrl' (CYRILLIC) */
const LETag dsrtScriptTag = 0x64737274; /* 'dsrt' (DESERET) */
const LETag devaScriptTag = 0x64657661; /* 'deva' (DEVANAGARI) */
const LETag dev2ScriptTag = 0x64657632; /* 'dev2' (DEVANAGARI v.2) (manually added) */
const LETag ethiScriptTag = 0x65746869; /* 'ethi' (ETHIOPIC) */
const LETag georScriptTag = 0x67656F72; /* 'geor' (GEORGIAN) */
const LETag gothScriptTag = 0x676F7468; /* 'goth' (GOTHIC) */
const LETag grekScriptTag = 0x6772656B; /* 'grek' (GREEK) */
const LETag gujrScriptTag = 0x67756A72; /* 'gujr' (GUJARATI) */
const LETag gjr2ScriptTag = 0x676A7232; /* 'gjr2' (GUJARATI v.2) (manually added) */
const LETag guruScriptTag = 0x67757275; /* 'guru' (GURMUKHI) */
const LETag gur2ScriptTag = 0x67757232; /* 'gur2' (GURMUKHI v.2) (manually added) */
const LETag haniScriptTag = 0x68616E69; /* 'hani' (HAN) */
const LETag hangScriptTag = 0x68616E67; /* 'hang' (HANGUL) */
const LETag hebrScriptTag = 0x68656272; /* 'hebr' (HEBREW) */
const LETag hiraScriptTag = 0x68697261; /* 'hira' (HIRAGANA) */
const LETag kndaScriptTag = 0x6B6E6461; /* 'knda' (KANNADA) */
const LETag knd2ScriptTag = 0x6B6E6432; /* 'knd2' (KANNADA v.2) (manually added) */
const LETag kanaScriptTag = 0x6B616E61; /* 'kana' (KATAKANA) */
const LETag khmrScriptTag = 0x6B686D72; /* 'khmr' (KHMER) */
const LETag laooScriptTag = 0x6C616F20; /* 'lao ' (LAO) */
const LETag latnScriptTag = 0x6C61746E; /* 'latn' (LATIN) */
const LETag mlymScriptTag = 0x6D6C796D; /* 'mlym' (MALAYALAM) */
const LETag mlm2ScriptTag = 0x6D6C6D32; /* 'mlm2' (MALAYALAM v.2) (manually added) */
const LETag mongScriptTag = 0x6D6F6E67; /* 'mong' (MONGOLIAN) */
const LETag mymrScriptTag = 0x6D796D72; /* 'mymr' (MYANMAR) */
const LETag ogamScriptTag = 0x6F67616D; /* 'ogam' (OGHAM) */
const LETag italScriptTag = 0x6974616C; /* 'ital' (OLD_ITALIC) */
const LETag oryaScriptTag = 0x6F727961; /* 'orya' (ORIYA) */
const LETag ory2ScriptTag = 0x6F727932; /* 'ory2' (ORIYA v.2) (manually added) */
const LETag runrScriptTag = 0x72756E72; /* 'runr' (RUNIC) */
const LETag sinhScriptTag = 0x73696E68; /* 'sinh' (SINHALA) */
const LETag syrcScriptTag = 0x73797263; /* 'syrc' (SYRIAC) */
const LETag tamlScriptTag = 0x74616D6C; /* 'taml' (TAMIL) */
const LETag tml2ScriptTag = 0x746D6C32; /* 'tml2' (TAMIL v.2) (manually added) */
const LETag teluScriptTag = 0x74656C75; /* 'telu' (TELUGU) */
const LETag tel2ScriptTag = 0x74656C32; /* 'tel2' (TELUGU v.2) (manually added) */
const LETag thaaScriptTag = 0x74686161; /* 'thaa' (THAANA) */
const LETag thaiScriptTag = 0x74686169; /* 'thai' (THAI) */
const LETag tibtScriptTag = 0x74696274; /* 'tibt' (TIBETAN) */
const LETag cansScriptTag = 0x63616E73; /* 'cans' (CANADIAN_ABORIGINAL) */
const LETag yiiiScriptTag = 0x79692020; /* 'yi  ' (YI) */
const LETag tglgScriptTag = 0x74676C67; /* 'tglg' (TAGALOG) */
const LETag hanoScriptTag = 0x68616E6F; /* 'hano' (HANUNOO) */
const LETag buhdScriptTag = 0x62756864; /* 'buhd' (BUHID) */
const LETag tagbScriptTag = 0x74616762; /* 'tagb' (TAGBANWA) */
const LETag braiScriptTag = 0x62726169; /* 'brai' (BRAILLE) */
const LETag cprtScriptTag = 0x63707274; /* 'cprt' (CYPRIOT) */
const LETag limbScriptTag = 0x6C696D62; /* 'limb' (LIMBU) */
const LETag linbScriptTag = 0x6C696E62; /* 'linb' (LINEAR_B) */
const LETag osmaScriptTag = 0x6F736D61; /* 'osma' (OSMANYA) */
const LETag shawScriptTag = 0x73686177; /* 'shaw' (SHAVIAN) */
const LETag taleScriptTag = 0x74616C65; /* 'tale' (TAI_LE) */
const LETag ugarScriptTag = 0x75676172; /* 'ugar' (UGARITIC) */
const LETag hrktScriptTag = 0x68726B74; /* 'hrkt' (KATAKANA_OR_HIRAGANA) */
const LETag bugiScriptTag = 0x62756769; /* 'bugi' (BUGINESE) */
const LETag glagScriptTag = 0x676C6167; /* 'glag' (GLAGOLITIC) */
const LETag kharScriptTag = 0x6B686172; /* 'khar' (KHAROSHTHI) */
const LETag syloScriptTag = 0x73796C6F; /* 'sylo' (SYLOTI_NAGRI) */
const LETag taluScriptTag = 0x74616C75; /* 'talu' (NEW_TAI_LUE) */
const LETag tfngScriptTag = 0x74666E67; /* 'tfng' (TIFINAGH) */
const LETag xpeoScriptTag = 0x7870656F; /* 'xpeo' (OLD_PERSIAN) */
const LETag baliScriptTag = 0x62616C69; /* 'bali' (BALINESE) */
const LETag batkScriptTag = 0x6261746B; /* 'batk' (BATAK) */
const LETag blisScriptTag = 0x626C6973; /* 'blis' (BLIS) */
const LETag brahScriptTag = 0x62726168; /* 'brah' (BRAHMI) */
const LETag chamScriptTag = 0x6368616D; /* 'cham' (CHAM) */
const LETag cirtScriptTag = 0x63697274; /* 'cirt' (CIRT) */
const LETag cyrsScriptTag = 0x63797273; /* 'cyrs' (CYRS) */
const LETag egydScriptTag = 0x65677964; /* 'egyd' (EGYD) */
const LETag egyhScriptTag = 0x65677968; /* 'egyh' (EGYH) */
const LETag egypScriptTag = 0x65677970; /* 'egyp' (EGYPTIAN_HIEROGLYPHS) */
const LETag geokScriptTag = 0x67656F6B; /* 'geok' (GEOK) */
const LETag hansScriptTag = 0x68616E73; /* 'hans' (HANS) */
const LETag hantScriptTag = 0x68616E74; /* 'hant' (HANT) */
const LETag hmngScriptTag = 0x686D6E67; /* 'hmng' (HMNG) */
const LETag hungScriptTag = 0x68756E67; /* 'hung' (HUNG) */
const LETag indsScriptTag = 0x696E6473; /* 'inds' (INDS) */
const LETag javaScriptTag = 0x6A617661; /* 'java' (JAVANESE) */
const LETag kaliScriptTag = 0x6B616C69; /* 'kali' (KAYAH_LI) */
const LETag latfScriptTag = 0x6C617466; /* 'latf' (LATF) */
const LETag latgScriptTag = 0x6C617467; /* 'latg' (LATG) */
const LETag lepcScriptTag = 0x6C657063; /* 'lepc' (LEPCHA) */
const LETag linaScriptTag = 0x6C696E61; /* 'lina' (LINA) */
const LETag mandScriptTag = 0x6D616E64; /* 'mand' (MANDAIC) */
const LETag mayaScriptTag = 0x6D617961; /* 'maya' (MAYA) */
const LETag meroScriptTag = 0x6D65726F; /* 'mero' (MEROITIC_HIEROGLYPHS) */
const LETag nkooScriptTag = 0x6E6B6F20; /* 'nko ' (NKO) */
const LETag orkhScriptTag = 0x6F726B68; /* 'orkh' (OLD_TURKIC) */
const LETag permScriptTag = 0x7065726D; /* 'perm' (PERM) */
const LETag phagScriptTag = 0x70686167; /* 'phag' (PHAGS_PA) */
const LETag phnxScriptTag = 0x70686E78; /* 'phnx' (PHOENICIAN) */
const LETag plrdScriptTag = 0x706C7264; /* 'plrd' (MIAO) */
const LETag roroScriptTag = 0x726F726F; /* 'roro' (RORO) */
const LETag saraScriptTag = 0x73617261; /* 'sara' (SARA) */
const LETag syreScriptTag = 0x73797265; /* 'syre' (SYRE) */
const LETag syrjScriptTag = 0x7379726A; /* 'syrj' (SYRJ) */
const LETag syrnScriptTag = 0x7379726E; /* 'syrn' (SYRN) */
const LETag tengScriptTag = 0x74656E67; /* 'teng' (TENG) */
const LETag vaiiScriptTag = 0x76616920; /* 'vai ' (VAI) */
const LETag vispScriptTag = 0x76697370; /* 'visp' (VISP) */
const LETag xsuxScriptTag = 0x78737578; /* 'xsux' (CUNEIFORM) */
const LETag zxxxScriptTag = 0x7A787878; /* 'zxxx' (ZXXX) */
const LETag zzzzScriptTag = 0x7A7A7A7A; /* 'zzzz' (UNKNOWN) */
const LETag cariScriptTag = 0x63617269; /* 'cari' (CARIAN) */
const LETag jpanScriptTag = 0x6A70616E; /* 'jpan' (JPAN) */
const LETag lanaScriptTag = 0x6C616E61; /* 'lana' (TAI_THAM) */
const LETag lyciScriptTag = 0x6C796369; /* 'lyci' (LYCIAN) */
const LETag lydiScriptTag = 0x6C796469; /* 'lydi' (LYDIAN) */
const LETag olckScriptTag = 0x6F6C636B; /* 'olck' (OL_CHIKI) */
const LETag rjngScriptTag = 0x726A6E67; /* 'rjng' (REJANG) */
const LETag saurScriptTag = 0x73617572; /* 'saur' (SAURASHTRA) */
const LETag sgnwScriptTag = 0x73676E77; /* 'sgnw' (SGNW) */
const LETag sundScriptTag = 0x73756E64; /* 'sund' (SUNDANESE) */
const LETag moonScriptTag = 0x6D6F6F6E; /* 'moon' (MOON) */
const LETag mteiScriptTag = 0x6D746569; /* 'mtei' (MEETEI_MAYEK) */
const LETag armiScriptTag = 0x61726D69; /* 'armi' (IMPERIAL_ARAMAIC) */
const LETag avstScriptTag = 0x61767374; /* 'avst' (AVESTAN) */
const LETag cakmScriptTag = 0x63616B6D; /* 'cakm' (CHAKMA) */
const LETag koreScriptTag = 0x6B6F7265; /* 'kore' (KORE) */
const LETag kthiScriptTag = 0x6B746869; /* 'kthi' (KAITHI) */
const LETag maniScriptTag = 0x6D616E69; /* 'mani' (MANI) */
const LETag phliScriptTag = 0x70686C69; /* 'phli' (INSCRIPTIONAL_PAHLAVI) */
const LETag phlpScriptTag = 0x70686C70; /* 'phlp' (PHLP) */
const LETag phlvScriptTag = 0x70686C76; /* 'phlv' (PHLV) */
const LETag prtiScriptTag = 0x70727469; /* 'prti' (INSCRIPTIONAL_PARTHIAN) */
const LETag samrScriptTag = 0x73616D72; /* 'samr' (SAMARITAN) */
const LETag tavtScriptTag = 0x74617674; /* 'tavt' (TAI_VIET) */
const LETag zmthScriptTag = 0x7A6D7468; /* 'zmth' (ZMTH) */
const LETag zsymScriptTag = 0x7A73796D; /* 'zsym' (ZSYM) */
const LETag bamuScriptTag = 0x62616D75; /* 'bamu' (BAMUM) */
const LETag lisuScriptTag = 0x6C697375; /* 'lisu' (LISU) */
const LETag nkgbScriptTag = 0x6E6B6762; /* 'nkgb' (NKGB) */
const LETag sarbScriptTag = 0x73617262; /* 'sarb' (OLD_SOUTH_ARABIAN) */
const LETag bassScriptTag = 0x62617373; /* 'bass' (BASS) */
const LETag duplScriptTag = 0x6475706C; /* 'dupl' (DUPL) */
const LETag elbaScriptTag = 0x656C6261; /* 'elba' (ELBA) */
const LETag granScriptTag = 0x6772616E; /* 'gran' (GRAN) */
const LETag kpelScriptTag = 0x6B70656C; /* 'kpel' (KPEL) */
const LETag lomaScriptTag = 0x6C6F6D61; /* 'loma' (LOMA) */
const LETag mendScriptTag = 0x6D656E64; /* 'mend' (MEND) */
const LETag mercScriptTag = 0x6D657263; /* 'merc' (MEROITIC_CURSIVE) */
const LETag narbScriptTag = 0x6E617262; /* 'narb' (NARB) */
const LETag nbatScriptTag = 0x6E626174; /* 'nbat' (NBAT) */
const LETag palmScriptTag = 0x70616C6D; /* 'palm' (PALM) */
const LETag sindScriptTag = 0x73696E64; /* 'sind' (SIND) */
const LETag waraScriptTag = 0x77617261; /* 'wara' (WARA) */
const LETag afakScriptTag = 0x6166616B; /* 'afak' (AFAK) */
const LETag jurcScriptTag = 0x6A757263; /* 'jurc' (JURC) */
const LETag mrooScriptTag = 0x6D726F6F; /* 'mroo' (MROO) */
const LETag nshuScriptTag = 0x6E736875; /* 'nshu' (NSHU) */
const LETag shrdScriptTag = 0x73687264; /* 'shrd' (SHARADA) */
const LETag soraScriptTag = 0x736F7261; /* 'sora' (SORA_SOMPENG) */
const LETag takrScriptTag = 0x74616B72; /* 'takr' (TAKRI) */
const LETag tangScriptTag = 0x74616E67; /* 'tang' (TANG) */
const LETag woleScriptTag = 0x776F6C65; /* 'wole' (WOLE) */
const LETag khojScriptTag = 0x6B686F6A; /* 'khoj' (KHOJ) */
const LETag tirhScriptTag = 0x74697268; /* 'tirh' (TIRH) */

const LETag nullScriptTag = 0x00000000; /* ''     (NULL) */


const LETag nullLanguageTag = 0x00000000; /* '' (null) */
const LETag araLanguageTag = 0x41524120; /* 'ARA' (Arabic) */
const LETag asmLanguageTag = 0x41534D20; /* 'ASM' (Assamese) */
const LETag benLanguageTag = 0x42454E20; /* 'BEN' (Bengali) */
const LETag farLanguageTag = 0x46415220; /* 'FAR' (Farsi) */
const LETag gujLanguageTag = 0x47554A20; /* 'GUJ' (Gujarati) */
const LETag hinLanguageTag = 0x48494E20; /* 'HIN' (Hindi) */
const LETag iwrLanguageTag = 0x49575220; /* 'IWR' (Hebrew) */
const LETag jiiLanguageTag = 0x4A494920; /* 'JII' (Yiddish) */
const LETag janLanguageTag = 0x4A414E20; /* 'JAN' (Japanese) */
const LETag kanLanguageTag = 0x4B414E20; /* 'KAN' (Kannada) */
const LETag kokLanguageTag = 0x4B4F4B20; /* 'KOK' (Konkani) */
const LETag korLanguageTag = 0x4B4F5220; /* 'KOR' (Korean) */
const LETag kshLanguageTag = 0x4B534820; /* 'KSH' (Kashmiri) */
const LETag malLanguageTag = 0x4D414C20; /* 'MAL' (Malayalam (Traditional)) */
const LETag marLanguageTag = 0x4D415220; /* 'MAR' (Marathi) */
const LETag mlrLanguageTag = 0x4D4C5220; /* 'MLR' (Malayalam (Reformed)) */
const LETag mniLanguageTag = 0x4D4E4920; /* 'MNI' (Manipuri) */
const LETag oriLanguageTag = 0x4F524920; /* 'ORI' (Oriya) */
const LETag sanLanguageTag = 0x53414E20; /* 'SAN' (Sanscrit) */
const LETag sndLanguageTag = 0x534E4420; /* 'SND' (Sindhi) */
const LETag snhLanguageTag = 0x534E4820; /* 'SNH' (Sinhalese) */
const LETag syrLanguageTag = 0x53595220; /* 'SYR' (Syriac) */
const LETag tamLanguageTag = 0x54414D20; /* 'TAM' (Tamil) */
const LETag telLanguageTag = 0x54454C20; /* 'TEL' (Telugu) */
const LETag thaLanguageTag = 0x54484120; /* 'THA' (Thai) */
const LETag urdLanguageTag = 0x55524420; /* 'URD' (Urdu) */
const LETag zhpLanguageTag = 0x5A485020; /* 'ZHP' (Chinese (Phonetic)) */
const LETag zhsLanguageTag = 0x5A485320; /* 'ZHS' (Chinese (Simplified)) */
const LETag zhtLanguageTag = 0x5A485420; /* 'ZHT' (Chinese (Traditional)) */
const LETag afkLanguageTag = 0x41464B20; /* 'AFK' (Afrikaans) */
const LETag belLanguageTag = 0x42454C20; /* 'BEL' (Belarussian) */
const LETag bgrLanguageTag = 0x42475220; /* 'BGR' (Bulgarian) */
const LETag catLanguageTag = 0x43415420; /* 'CAT' (Catalan) */
const LETag cheLanguageTag = 0x43484520; /* 'CHE' (Chechen) */
const LETag copLanguageTag = 0x434F5020; /* 'COP' (Coptic) */
const LETag csyLanguageTag = 0x43535920; /* 'CSY' (Czech) */
const LETag danLanguageTag = 0x44414E20; /* 'DAN' (Danish) */
const LETag deuLanguageTag = 0x44455520; /* 'DEU' (German) */
const LETag dznLanguageTag = 0x445A4E20; /* 'DZN' (Dzongkha) */
const LETag ellLanguageTag = 0x454C4C20; /* 'ELL' (Greek) */
const LETag engLanguageTag = 0x454E4720; /* 'ENG' (English) */
const LETag espLanguageTag = 0x45535020; /* 'ESP' (Spanish) */
const LETag etiLanguageTag = 0x45544920; /* 'ETI' (Estonian) */
const LETag euqLanguageTag = 0x45555120; /* 'EUQ' (Basque) */
const LETag finLanguageTag = 0x46494E20; /* 'FIN' (Finnish) */
const LETag fraLanguageTag = 0x46524120; /* 'FRA' (French) */
const LETag gaeLanguageTag = 0x47414520; /* 'GAE' (Gaelic) */
const LETag hauLanguageTag = 0x48415520; /* 'HAU' (Hausa) */
const LETag hrvLanguageTag = 0x48525620; /* 'HRV' (Croation) */
const LETag hunLanguageTag = 0x48554E20; /* 'HUN' (Hungarian) */
const LETag hyeLanguageTag = 0x48594520; /* 'HYE' (Armenian) */
const LETag indLanguageTag = 0x494E4420; /* 'IND' (Indonesian) */
const LETag itaLanguageTag = 0x49544120; /* 'ITA' (Italian) */
const LETag khmLanguageTag = 0x4B484D20; /* 'KHM' (Khmer) */
const LETag mngLanguageTag = 0x4D4E4720; /* 'MNG' (Mongolian) */
const LETag mtsLanguageTag = 0x4D545320; /* 'MTS' (Maltese) */
const LETag nepLanguageTag = 0x4E455020; /* 'NEP' (Nepali) */
const LETag nldLanguageTag = 0x4E4C4420; /* 'NLD' (Dutch) */
const LETag pasLanguageTag = 0x50415320; /* 'PAS' (Pashto) */
const LETag plkLanguageTag = 0x504C4B20; /* 'PLK' (Polish) */
const LETag ptgLanguageTag = 0x50544720; /* 'PTG' (Portuguese) */
const LETag romLanguageTag = 0x524F4D20; /* 'ROM' (Romanian) */
const LETag rusLanguageTag = 0x52555320; /* 'RUS' (Russian) */
const LETag skyLanguageTag = 0x534B5920; /* 'SKY' (Slovak) */
const LETag slvLanguageTag = 0x534C5620; /* 'SLV' (Slovenian) */
const LETag sqiLanguageTag = 0x53514920; /* 'SQI' (Albanian) */
const LETag srbLanguageTag = 0x53524220; /* 'SRB' (Serbian) */
const LETag sveLanguageTag = 0x53564520; /* 'SVE' (Swedish) */
const LETag tibLanguageTag = 0x54494220; /* 'TIB' (Tibetan) */
const LETag trkLanguageTag = 0x54524B20; /* 'TRK' (Turkish) */
const LETag welLanguageTag = 0x57454C20; /* 'WEL' (Welsh) */


U_NAMESPACE_END
#endif
